/****************************************************************************
    Xplanetbg 0.90 - run Xplanet in the background
    Copyright (C) 2001 Hari Nair <hari@alumni.caltech.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
****************************************************************************/

// This has to come before the other includes on linux, otherwise
// strptime won't be found (?)
#include "checkfuncs.h" 

#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <iostream>
using namespace std;

#include <unistd.h>

#include "orbit.h"
#include "xp_getopt.h"
#include "xplanetbg.h"

#ifdef HAVE_STRPTIME
#include <sys/time.h>
struct timeval tval;
float timewarp = 1;
int use_date = 0;
#endif

string title;
string base, extension;

bool debug = false;
string prev_command;
string post_command;
int nice_value = 0;
int num_times = 0;
double rotation = 0;

struct s_orbit_info {
    double pos_lat, pos_long, duration, inclination;
};

bool fullscreen = false;

#ifdef HAVE_LIBX11

#include <X11/Xlib.h>
#include <X11/Xutil.h>

Display *display = NULL;
Window window = 0;
int window_mask;                // geometry flags
int x=0, y=0;                   // Window position
unsigned int width = 512;
unsigned int height = 512;

Window
open_window()
{
    Window root;
  
    display = XOpenDisplay(NULL);
    if (display == NULL)
    { 
        cerr << "Can't open displaylay\n";
        exit(EXIT_FAILURE);
    }

    int screen_num = DefaultScreen(display);
    root = RootWindow(display, screen_num);

    if (fullscreen)
    {
	width = DisplayWidth(display, screen_num);
	height = DisplayHeight(display, screen_num);
    }

    if (window_mask & XNegative) 
        x += (DisplayWidth(display, screen_num) - width);
    if (window_mask & YNegative) 
        y += (DisplayHeight(display, screen_num) - height);
  
    window = XCreateSimpleWindow(display, root, x, y, width, height, 4, 
                                 WhitePixel (display, screen_num),
                                 BlackPixel (display, screen_num));

    if ((window_mask & XValue) || (window_mask & YValue)
        || (window_mask & XNegative) || (window_mask & YNegative))
    {
        XSizeHints *hints = XAllocSizeHints();
        hints->flags = USPosition;
        XSetWMNormalHints(display, window, hints);
    }

    XTextProperty windowName;
    
    const char *temp;
    if (title.empty())
	temp = versionstring.c_str();
    else
	temp = title.c_str();
    
    XStringListToTextProperty((char **) &temp, 1, &windowName);
    XSetWMName(display, window, &windowName);
    XSync(display, True);
    return(window);
}
#else
extern "C" {
    extern int 
    XParseGeometry(const char *string, int *x, int *y, unsigned int *width, 
                   unsigned int *height);
}
#endif // HAVE_LIBX11

void show_help()
{
    cout << "valid options:\n";
    cout << "\t-ba[ckground]         image_file (string)\n";
    cout << "\t-bl[end]                                 \n";
    cout << "\t-bo[dy]               body       (string)\n";
    cout << "\t-ce[nter]             x,y        (int, int)\n";
    cout << "\t-cloud_i[mage]        image_file (string)\n";
    cout << "\t-cloud_s[sec]         image_file (string)\n";
    cout << "\t-cloud_t[hreshold]    threshold  (int)\n";
    cout << "\t-col[or]              colorname  (string)\n";
    cout << "\t-dat[e]               string     (string)\n";
    cout << "\t-day[side]\n";
    cout << "\t-demf[ile]            demfile    (string)\n";
    cout << "\t-dems[cale]           scale      (float)\n";
    cout << "\t-e[arthside]\n";
    cout << "\t-fo[nt]               fontname   (string)\n";
    cout << "\t-fontd[ir]            directory  (string)\n";
    cout << "\t-fonts[ize]           size       (int)\n";
    cout << "\t-ful[lscreen]\n";
    cout << "\t-fu[zz]               pixels     (int)   \n";
    cout << "\t-ge[ometry]           string     (string)\n";
    cout << "\t-gmtlabel\n";
    cout << "\t-greatarcfile         filename   (string)\n";
    cout << "\t-grid\n";
    cout << "\t-grid1                spacing    (int)\n";
    cout << "\t-grid2                spacing    (int)\n";
    cout << "\t-h[elp]\n";
    cout << "\t-i[mage]              image_file (string)\n";
    cout << "\t-label                                   \n";
    cout << "\t-labeln[ame]\n";
    cout << "\t-labelp[os]           string     (string)\n";
    cout << "\t-lat[itude]           degrees    (float) \n";
    cout << "\t-loc[altime]          time       (float) \n";
    cout << "\t-lon[gitude]          degrees    (float) \n";
    cout << "\t-map[dir]             directory  (string)\n";
    cout << "\t-markerb[ounds]       filename   (string)\n";
    cout << "\t-markerf[ile]         filename   (string)\n";
    cout << "\t-markers                                 \n";
    cout << "\t-mo[onside]\n";
    cout << "\t-nice                 priority   (int)\n";
    cout << "\t-night_[image]        image_file (string)\n";
    cout << "\t-nights[ide]\n";
    cout << "\t-no[transparency]\n";
    cout << "\t-nu[m_times]          num_times  (int)   \n";
    cout << "\t-ob[server]           lon,lat    (float, float)\n";
    cout << "\t-orb[it] duration:inclination    (float):(float)\n";
    cout << "\t-ou[tput]             filename   (string)\n";
    cout << "\t-po[st_command]            command    (string)\n";
    cout << "\t-pre[v_command]            command    (string)\n";
    cout << "\t-pr[ojection]         type       (string)\n";
    cout << "\t-q[uality]            quality    (int)\n";
    cout << "\t-rad[ius]             percent    (int)   \n";
    cout << "\t-rand[om]                                \n";
    cout << "\t-rang[e]              range      (float) \n";
    cout << "\t-roo[t]\n";
    cout << "\t-rot[ate]             degrees    (float) \n";
    cout << "\t-satf[ile]            filename   (string)\n";
    cout << "\t-satt[rackid]               id   (string)\n";
    cout << "\t-sh[ade]              percent    (int)   \n";
    cout << "\t-st[arfreq]           density    (float) \n";
    cout << "\t-su[nrel]     del_lon,del_lat    (float, float)\n";
    cout << "\t-sw[ap]\n";
    cout << "\t-te[rminator]        terminator  (string)\n";
    cout << "\t-ti[mewarp]           factor     (float) \n";
    cout << "\t-tra[nspng]\n";
    cout << "\t-tru[etype]\n";
    cout << "\t-v[ersion]\n";
    cout << "\t-wa[it]               seconds    (int)   \n";
    cout << "\t-wi[ndow]\n";
    cout << "\t-x[screensaver]\n";
    cout << "\nCharacters inside the square braces are optional\n";
    exit (EXIT_SUCCESS);
}

void 
process_args (string &command, int argc, char **argv, int &wait,
              struct s_orbit_info &orbit_info)
{
    enum optionType
    {
        DATE, DAYSIDE, DEBUG, EARTHSIDE, FULLSCREEN, GEOMETRY, 
	HELP, LATITUDE, LONGITUDE, MOONSIDE,
        NICE, NIGHTSIDE, NUM_TIMES, ORBIT, OUTPUT, PASS0, PASS1, 
	POSTCOMMAND, PREVCOMMAND, ROTATE, 
	TIMEWARP, VERSION, WAIT, WINDOW  
    };
  
    static struct option long_options[] =
    {
        {"background",      required_argument, NULL, PASS1},
        {"blend",           no_argument,       NULL, PASS0},
        {"body",            required_argument, NULL, PASS1},
        {"center",          required_argument, NULL, PASS1},
        {"cloud_gamma",     required_argument, NULL, PASS1},
        {"cloud_image",     required_argument, NULL, PASS1},
        {"cloud_ssec",      required_argument, NULL, PASS1},
        {"cloud_threshold", required_argument, NULL, PASS1},
        {"color",           required_argument, NULL, PASS1},
        {"date",            required_argument, NULL, DATE},
        {"dayside",         no_argument,       NULL, DAYSIDE},
        {"debug",           no_argument,       NULL, DEBUG},
        {"demfile",         required_argument, NULL, PASS1},
        {"demscale",        required_argument, NULL, PASS1},
        {"earthside",       no_argument,       NULL, EARTHSIDE},
        {"font",            required_argument, NULL, PASS1},
        {"fontdir",         required_argument, NULL, PASS1},
        {"fontsize",        required_argument, NULL, PASS1},
        {"fullscreen",      no_argument,       NULL, FULLSCREEN},
        {"fuzz",            required_argument, NULL, PASS1},
        {"geometry",        required_argument, NULL, GEOMETRY},
        {"gmtlabel",        no_argument,       NULL, PASS0},
        {"greatarcfile",    required_argument, NULL, PASS1},
        {"grid",            no_argument,       NULL, PASS0},
        {"grid1",           required_argument, NULL, PASS1},
        {"grid2",           required_argument, NULL, PASS1},
        {"help",            no_argument,       NULL, HELP},
        {"image",           required_argument, NULL, PASS1},
        {"label",           no_argument,       NULL, PASS0},
        {"labelname",       no_argument,       NULL, PASS0},
        {"labelpos",        required_argument, NULL, PASS1},
        {"latitude",        required_argument, NULL, LATITUDE},
        {"localtime",       required_argument, NULL, PASS1},
        {"longitude",       required_argument, NULL, LONGITUDE},
        {"mapdir",          required_argument, NULL, PASS1},
        {"markerbounds",    required_argument, NULL, PASS1},
        {"markerfile",      required_argument, NULL, PASS1},
        {"markers",         no_argument,       NULL, PASS0},
        {"moonside",        no_argument,       NULL, MOONSIDE},
        {"nice",            required_argument, NULL, NICE},
        {"night_image",     required_argument, NULL, PASS1},
        {"nightside",       no_argument,       NULL, NIGHTSIDE},
        {"notransparency",  no_argument,       NULL, PASS0},
        {"num_times",       required_argument, NULL, NUM_TIMES},
        {"observer",        required_argument, NULL, PASS1},
        {"orbit",           required_argument, NULL, ORBIT},
        {"output",          required_argument, NULL, OUTPUT},
        {"post_command",    required_argument, NULL, POSTCOMMAND},
        {"prev_command",    required_argument, NULL, PREVCOMMAND},
        {"projection",      required_argument, NULL, PASS1},
        {"quality",         required_argument, NULL, PASS1},
        {"radius",          required_argument, NULL, PASS1},
        {"random",          no_argument,       NULL, PASS0},
        {"range",           required_argument, NULL, PASS1},
        {"root",            no_argument,       NULL, PASS0},
        {"rotate",          required_argument, NULL, ROTATE},
        {"satfile",         required_argument, NULL, PASS1},
        {"sattrackid",      required_argument, NULL, PASS1},
        {"shade",           required_argument, NULL, PASS1},
        {"starfreq",        required_argument, NULL, PASS1},
        {"sunrel",          required_argument, NULL, PASS1},
        {"swap",            no_argument,       NULL, PASS0},
        {"terminator",      required_argument, NULL, PASS1},
        {"timewarp",        required_argument, NULL, TIMEWARP},
        {"transpng",        no_argument,       NULL, PASS1},
        {"truetype",        no_argument,       NULL, PASS0},
        {"version",         no_argument,       NULL, VERSION},
        {"wait",            required_argument, NULL, WAIT},
        {"window",          no_argument,       NULL, WINDOW},
        {"xscreensaver",    no_argument,       NULL, PASS0},
        {NULL,              0,                 NULL, 0}
    };

    int this_option;
    int option_index = 0;
    while((this_option = xp_getopt_long_only(argc, argv, "+", long_options, 
                                             &option_index)) >= 0)
    {
        switch (this_option)
        {
        case DATE:
        {
#ifdef HAVE_STRPTIME
            use_date = 1;
            struct tm tm = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
            strptime(optarg, "%d %b %Y %T", &tm);
            tval.tv_sec = mktime(&tm);
            if (tm.tm_isdst > 0) tval.tv_sec -= 3600;
#else
            cerr << "This system does not have the strptime function.\n";
            cerr << "The -date option will be ignored.\n";
#endif
        }
        break;
        case DAYSIDE:
        {
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
            title = "Dayside view";
        }
        break;
	case DEBUG:
	    debug = true;
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
	    break;
        case EARTHSIDE:
        {
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
            title = "View from earth";
        }
        break;
        case FULLSCREEN:
	    fullscreen = true;
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
#ifdef HAVE_LIBX11
            window = 1;
#endif
            break;
        case GEOMETRY:
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
	    command += optarg;
	    command += " ";
#ifdef HAVE_LIBX11
            window_mask = XParseGeometry(optarg, &x, &y, &width, &height);
            window = 1;
#endif
            break;
        case HELP:
            show_help();
            break;
        case LATITUDE:
        {
            double obslat;
            sscanf(optarg, "%lf", &obslat);
            if (obslat < -90 || obslat > 90) 
            {
                if (obslat < -90) obslat = -90;
                if (obslat > 90) obslat = 90;
            }
            orbit_info.pos_lat = obslat;
        }
        break;
        case LONGITUDE:
        {
            double obslon;
            sscanf(optarg, "%lf", &obslon);
            if (obslon < 0 || obslon > 360) 
            {
                obslon = fmod(obslon, 360.);
                if (obslon < 0) obslon += 360;
            }
            orbit_info.pos_long = obslon;
        } 
        break;
        case MOONSIDE:
        {
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
            title = "View from the moon";
        }
        break;
        case NICE:
            sscanf(optarg, "%d", &nice_value);
            if (nice_value < 0 || nice_value > 19) 
            {
                cerr << "-nice must take an argument between 0 and 19\n";
                exit(EXIT_FAILURE);
            }
            break;
        case NIGHTSIDE:
        {
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
            title = "Nightside view";
        }
        break;
        case NUM_TIMES:
            sscanf(optarg, "%d", &num_times);
            if (num_times < 1) 
            {
                cerr << "-num_times must be positive\n";
                exit (EXIT_FAILURE);
            }
            break;
        case ORBIT:
        {
            float duration, inclination;
            sscanf(optarg, "%f:%f", &duration, &inclination);
            if (duration < 0)
            {
                cerr << "orbit duration must be positive\n";
                exit(EXIT_FAILURE);
            }
            orbit_info.duration = duration;
            orbit_info.inclination = inclination;
        }
        break;
        case OUTPUT:
        {
	    base = optarg;
            if (base.find('.') == string::npos)
	    {
		extension = defaultmapext;
	    }
	    else
	    {
		extension.assign(base, base.find('.'), base.size());
		base.assign(base, 0, base.find('.'));
	    }

#ifdef HAVE_LIBX11          
            window = 0;
#endif
        }
        break;
        case PASS0:
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
            break;
        case PASS1:
	    command += "-";
	    command += long_options[option_index].name;
	    command += " ";
	    command += optarg;
	    command += " ";
            break;
	case PREVCOMMAND:
	    prev_command = optarg;
	    break;
	case POSTCOMMAND:
	    post_command = optarg;
	    break;
        case ROTATE:
        {
            double obsrot;
            sscanf(optarg, "%lf", &obsrot);
            if (obsrot < 0 || obsrot > 360) 
            {
                obsrot = fmod(obsrot, 360.);
                if (obsrot < 0) obsrot += 360;
            }
            rotation = obsrot;
        }
        break;
        case TIMEWARP:
#ifdef HAVE_STRPTIME
            use_date = 1;
            sscanf(optarg, "%f", &timewarp);
            if (timewarp == 0) 
            {
                cerr << "timewarp must be non-zero\n";
                exit(EXIT_FAILURE);
            }
#else
            cerr << "This system does not have the strptime function.\n";
            cerr << "The -timewarp option will be ignored.\n";
#endif
            break;
        case VERSION:
            cout << versionstring << endl;
            cout << "The latest version can be found at " 
                 << "http://xplanet.sourceforge.net\n";
            exit(EXIT_SUCCESS);
        case WAIT:
            sscanf(optarg, "%d", &wait);
            break;
        case WINDOW:
#ifdef HAVE_LIBX11
            window = 1;
#else
            cerr << "The -window option is not supported without X11.\n";
#endif
            break;
        default:
            cerr << "Use \"xplanetbg -help\" for a list of valid options\n";
            exit(EXIT_FAILURE);
        }
    }
  
    if (optind < argc)
    {
	cerr << "unrecognized options: ";
	while (optind < argc) cerr << argv[optind++] << " ";
	cerr << "\nPerhaps you didn't supply an argument to -"
	     << long_options[option_index].name << "?\n";
	exit(EXIT_FAILURE);
    }

    char cbuffer[64];

    /* If we haven't specified an orbit but we do have a 
       -longitude or -latitude argument */
    if (!orbit_info.duration)
    {
        if (orbit_info.pos_lat)
        {
            sprintf(cbuffer, "-latitude %5.1f ", orbit_info.pos_lat);
	    command += cbuffer;
        }
        if (orbit_info.pos_long)
        {
            sprintf(cbuffer, "-longitude %5.1f ", orbit_info.pos_long);
	    command += cbuffer;
        }
    }

#ifdef HAVE_LIBX11
    if (window) 
    {
        sprintf (cbuffer, "-XID %lu ", open_window());
	command += cbuffer;
    }
#endif
}

int 
main(int argc, char **argv)
{
    int wait = 300;  // time, in seconds, between updates

    // Initialize to current time
#ifdef HAVE_STRPTIME
    gettimeofday(&tval, NULL);
#endif

    string stat_command = "xplanet ";
    string dyn_command;
    char cbuffer[256];
    struct s_orbit_info obi = {0,0,0,0};
    process_args(stat_command, argc, argv, wait, obi);

    Orbit orb(obi.duration, obi.pos_lat, obi.pos_long, obi.inclination);
 
    int current_time = 0;
    while (1)
    {
        if (nice_value) 
        {
	    dyn_command = nicecmd;
            sprintf(cbuffer, "-%d ", nice_value);
	    dyn_command += cbuffer;
            dyn_command += stat_command;
        }
        else
            dyn_command = stat_command;

        if (num_times != 0) 
        {
            current_time++;
            if (current_time > num_times) break;
            if (!base.empty())
            {
                int digits = (int) (log10((double) num_times) + 1);
                sprintf(cbuffer, "-output %s%.*d%s ", base.c_str(),
			digits, current_time, extension.c_str());
                dyn_command += cbuffer;
            }
        }
        else
        {
            if (!base.empty())
            {
                sprintf(cbuffer, "-output %s%s ", base.c_str(), 
			extension.c_str());
                dyn_command += cbuffer;
            }
        }             
      
        if (obi.duration > 0)
        {
            double orblat = orb.Lat();
            double orblon = orb.Long();
            sprintf(cbuffer, "-latitude %5.1f ", orblat);
	    dyn_command += cbuffer;
            sprintf(cbuffer, "-longitude %5.1f ", orblon);
	    dyn_command += cbuffer;
            sprintf(cbuffer, "-rotate %5.1f", (rotation + obi.inclination 
                                               - orb.Theta()));
	    dyn_command += cbuffer;
        }
        else if (rotation != 0)
        {
            sprintf(cbuffer, "-rotate %5.1f", rotation);
	    dyn_command += cbuffer;
        }
      
#ifdef HAVE_STRPTIME
        if (use_date)
        {
            struct tm *tm_ptr = localtime((time_t *) &tval.tv_sec);
            strftime(cbuffer, sizeof(cbuffer), "%d %b %Y %T", tm_ptr);
	    dyn_command += " -date \"";
            dyn_command += cbuffer;
            dyn_command += "\" ";
            tval.tv_sec += (time_t) timewarp * wait;
        }
#endif

        if (debug || !base.empty()) 
            cout << dyn_command << endl;
	
	if (!prev_command.empty())
	{
	    if (system(prev_command.c_str()) != 0) 
	    {
		cerr << "Can't execute " << prev_command << endl;
	    }
	}
	    

        if (system(dyn_command.c_str()) != 0) 
	{
	    cerr << "Xplanetbg exiting\n";
	    break;
	}

	if (!post_command.empty())
	{
	    if (system(post_command.c_str()) != 0) 
	    {
		cerr << "Can't execute " << post_command << endl;
	    }
	}
	    
#ifdef HAVE_STRPTIME
        if (obi.duration > 0) orb.WaitTime(timewarp * wait/3600.0);
#endif
      
#ifdef HAVE_LIBX11
        if (window)
        {
            Atom wmDeleteWindow = XInternAtom(display, "WM_DELETE_WINDOW", 
                                              False);
            XSetWMProtocols(display, window, &wmDeleteWindow, 1);
            XSelectInput(display, window, KeyPressMask);
            XEvent event;
            for (int i = 0; i < wait * 10; i++)
            {
                if (XCheckTypedWindowEvent(display, window, ClientMessage, 
                                           &event) == True) 
                {
                    if ((unsigned int) event.xclient.data.l[0] 
                        == wmDeleteWindow)
                        exit(EXIT_SUCCESS);
                }
                else if (XCheckTypedWindowEvent(display, window, KeyPress, 
                                                &event) == True)
                {
                    KeySym keysym;
                    char keybuf;
                    XLookupString(&(event.xkey), &keybuf, 1, &keysym, NULL);
                    if (keybuf == 'q' || keybuf == 'Q') 
                        exit(EXIT_SUCCESS);
                }
                else
                    usleep(100000);   // sleep for 1/10 second
            }
        }
        else
#endif // HAVE_LIBX11
            sleep(wait);

    } 
    exit(EXIT_FAILURE);
}
